import logging

# IN GENERAL:

# specify index_file (list of filenames)
# or corpusfolder (all .txt in a directory)
# or both - corpusfolder is root directory, index_file is the rest of the path
# variables = which variables to include (see variable_creator module)
# database_name = name for database output file


# SPECIFIC DESCRIPTIONS:

# database_creator: the original, optimized for edgar filepaths


def database_creator(variables, database_name, index_file='', corpusfolder=''):
    import re, os, time, glob2
    import pandas as pd
    from sec_edgar_tools import variable_creators
    
    # list of file names
    if index_file != '':
        # index file (list of file names given, with optional corpus folder)
        with open(index_file, "r") as f:
            files = f.read().split('\n')
            files = [ corpusfolder+f for f in files if f != '' ]
    if index_file == '':
        if corpusfolder == '':
            logging.error('Hey buddy, specify an index_file or a corpus_folder (or both).')
            return
        # all .txt files in the corpus folder
        files =  glob2.glob(corpusfolder+'**/*.txt')

    # progress counter
    time_start, time_elapsed, i, l = time.time(), 1, 0, len(files)

    # progress counter
    i, l = 1, len(list(files))
    
    # initialize data frame
    df = pd.DataFrame()
        
    # loop through files in corpus folder
    for file in files:
        
        # construct data observation

        # filepath
        filepath = file
        data = {'filepath':filepath}
        
        # other variables
        with open(file, errors='ignore') as g:
            raw = g.read()
        for variable in variables:
            # make 'v' the function that is called "variable"
            # "variable" is located in 'variable_creators'
            # which is in the sec_edgar_tools module
            v = getattr(variable_creators, variable)
            data.update(v(raw=raw))

        # append constructed observation to full data
        df1 = pd.DataFrame([list(data.values())], columns=data.keys())
        df = df.append(df1, ignore_index=True)

        # progress update
        i+=1
        time_elapsed = time.time() - time_start
        time_remaining = (l-i)*(time_elapsed/i)/60
        p=100*i/l
        print('Time remaining: {0:.2f} minutes. Percent done: {1:.3f}        '.format(time_remaining, p), end='\r')

    print('\n')
    logging.info('Total time: {0:.2f} minutes. Rate: {1:.2f} files/sec'.format(time_elapsed/60, l/time_elapsed))
    df.to_csv(database_name, index=False)
