# # # DESCRIPTION # # #
# GIVEN     (1) `states.txt'
#               list of states (laws)
#           (2) `govlawhead.txt'
#               list of regex around which we expect to find an entry from (1)
#           (3) `filepaths.txt'
#               a list of absolute filepaths to contracts
# OUTPUT    (4) data.txt
#               a clean file: 'filepath,filename,docnum,choice_of_law'
#               choice of law for each file in (3)

$|=1;


# # # USER INPUT # # #
# if you want to use corpus folder, also uncomment below in loop
#$corpusfolder 	= '/Users/ssanga/sec-contracts/input/ex10_tokenized/';
#$corpusfolder = '/Users/ssanga/Dropbox/Code/SEC_EDGAR/';

# # # INITIALIZE # # #

# set directory (to folder with this script)
use File::Basename qw();
my ($name, $path, $suffix) = File::Basename::fileparse($0);
chdir $path;


# list of states, choice of law headers, file paths
open (IN, "states.txt"); @states=<IN>; close IN;
open (IN, "govlawhead.txt"); @govlawhead=<IN>; close IN;
open (IN, "../filepaths_allcontracts.txt"); @files=<IN>; close IN;

# progress counter
# total files
# my $file_count;
# opendir(my $dh, $corpusfolder) or die "opendir($corpusfolder): $!";
# while (my $de = readdir($dh)) { $file_count++; }
# closedir($dh);

my $counter=0;
# start time
my $start_time = time()-1;
my $file_count = @files;



# # # PARSE # # #

open (OUT, ">labels.txt");
print OUT "filepath,filename,docnum,col\n";
foreach $file (@files){
#for $file (<$corpusfolder/*>) {
	print OUT join(",",filepath(),filename(),docnum(),col())."\n";
	$counter++;
	my $time_elapsed = time() - $start_time;
	my $p = $counter/$file_count;
	my $time_remaining = ($file_count - $counter)*($time_elapsed/$counter);
	printf("\rFiles done: %.0f of %.0f. Time remaining: %.2f minutes", $counter,$file_count,$time_remaining/60);
}
my $parse_rate = $file_count/(time() - $start_time);
printf("\nRate: %.0f files/sec\n", $parse_rate);
close OUT;



# # # FUNCTIONS # # #

# contract ID
sub filepath {
	$x = $file;
	$x =~ s/^\s+|\s+$//g;
	return $x;
}

sub filename { 
	$file =~ m{edgar.*?/(.*)/.*?\.txt};
	$x = 'edgar/' . $1 . '.txt';
}

sub docnum { 
	$file =~ m{.*/(.*)\.txt};
	$x = $1;
}

# choice of law
sub col { 
	# no. of occurances of " Governing law ", or if available: the governing law state
	open (FH, "$file");
	my $lines = do { local $/; <FH> };
	close FH;
	my $thecount=0; my %statecount; my $s; my $match; my $anymatch=0;
	
	OUTER:	# label outer loop
	foreach $govlawhead (@govlawhead){ # cycle through governing law headers
		chomp($govlawhead);
		while ($lines =~ m/($govlawhead)/sg) {
			$govlawtext=$1;
			my %statecount; 				# reset values for current statecount and match indicator
			my $match;
			$thecount++; 					# total number of gov law clauses
		
			foreach $state (@states){
				chomp($state);
				if ($govlawtext =~ /(.*?)$state/is) {
					$statecount{$state}=length($1);
					$match=1;
					$anymatch=1;
				}
			}
			# strip off parts of the regex that you don't want to return
			if ($match==1) { 
				$s =  smallest_value(\%statecount); 
				$s =~ s/\.\{0,3\}/ /g; 
				$s =~ s/\.\{0,4\}/ /g; 
				$s =~ s/\+\(\\W\|\\n\)//g; 
				$s =~ s/\(\?\! Stock\)/ /g;
				$s =~ s/\[\^D\]/ /g;
				$s =~ s/^\s+//;
				$s =~ s/\s+$//;
			}
		}
		last OUTER if $anymatch==1; # break free from all loops if govlawheader finds a state	
	}
	if ($anymatch==0) { $s=$thecount; }
	return $s;
}

# smallest value of a hash
sub smallest_value (\%) {
    my $hash   = shift;
    my ($key, @keys) = keys   %$hash;
    my ($small, @vals) = values %$hash;

    for (0 .. $#keys) {
        if ($vals[$_] < $small) {
            $small = $vals[$_];
            $key = $keys[$_];
        }
    }
    $key;
}

