# # # DESCRIPTION # # #
# GIVEN     (1) `categories.txt'
#               list of contract categories, with keyword searches
#           (2) `filepaths.txt'
#               a list of absolute filepaths to contracts
# OUTPUT    (3) data.txt
#               a clean file: 'filepath,filename,docnum,category'
#               category for each file in (2)

$|=1;


# # # USER INPUT # # #
# if you want to use corpus folder, also uncomment below in loop
#$corpusfolder 	= '/Users/ssanga/sec-contracts/input/ex10_tokenized/';
#$corpusfolder = '/Users/ssanga/Dropbox/Code/SEC_EDGAR/';

# # # INITIALIZE # # #

# set directory (to folder with this script)
use File::Basename qw();
my ($name, $path, $suffix) = File::Basename::fileparse($0);
chdir $path;


# list of categories, file paths
my %categ; open (FH, "categories.txt"); while (my $line = <FH>) {
	$line =~ s/\t//g;
	$line =~ s/\s/.{0,3}/g;
	$line =~ /(.*?),(.*)/;
	my $key = $2; $categ{$key} = $1;
	#print "key-$key\n";
	#print "val-$categ{$key}\n"; 
} close FH;
open (IN, "../filepaths_allcontracts.txt"); @files=<IN>; close IN;

# progress counter
# total files
# my $file_count;
# opendir(my $dh, $corpusfolder) or die "opendir($corpusfolder): $!";
# while (my $de = readdir($dh)) { $file_count++; }
# closedir($dh);

my $counter=0;
# start time
my $start_time = time()-1;
my $file_count = @files;



# # # PARSE # # #

open (OUT, ">labels.txt");
print OUT "filepath,filename,docnum,category\n";
foreach $file (@files){
#for $file (<$corpusfolder/*>) {
	print OUT join(",",filepath(),filename(),docnum(),category())."\n";
	$counter++;
	my $time_elapsed = time() - $start_time;
	my $p = $counter/$file_count;
	my $time_remaining = ($file_count - $counter)*($time_elapsed/$counter);
	printf("\rFiles done: %.0f of %.0f. Time remaining: %.2f minutes", $counter,$file_count,$time_remaining/60);
}
my $parse_rate = $file_count/(time() - $start_time);
printf("\nRate: %.0f files/sec\n", $parse_rate);
close OUT;



# # # FUNCTIONS # # #

# contract ID
sub filepath {
	$x = $file;
	$x =~ s/^\s+|\s+$//g;
	return $x;
}

sub filename { 
	$file =~ m{edgar.*?/(.*)/.*?\.txt};
	$x = 'edgar/' . $1 . '.txt';
}

sub docnum { 
	$file =~ m{.*/(.*)\.txt};
	$x = $1;
}

# category
sub category { 
	# contract subject matter from header, see input/categories.txt
	open (FH, "$file"); local $/; my $lines = <FH>; close FH;
	$lines=substr($lines,0,1000);
	keys %categ; my %aaa; my $s;
	
	while(my($k, $v) = each %categ) {	# cycle though all keywords
		if ($lines =~ /(.*?)$k/is) { 	# if match keyword
			$aaa{$v} = length($1); 		# then assign value-category as key to hash that returns no. of chars until keyword
		} 
	}
	$s=smallest_value(\%aaa);
	return $s;
}

# smallest value of a hash
sub smallest_value (\%) {
    my $hash   = shift;
    my ($key, @keys) = keys   %$hash;
    my ($small, @vals) = values %$hash;

    for (0 .. $#keys) {
        if ($vals[$_] < $small) {
            $small = $vals[$_];
            $key = $keys[$_];
        }
    }
    $key;
}

